@@ -180,14 +180,16 @@ SEXP R_trace_glmboost(SEXP x, SEXP MPinv, SEXP xselect) {
     negative gradient of the partial likelihood of a Cox model
     see formula (4.65) in Chapter 4 of Greg Ridgeway's thesis    
     http://www.i-pensieri.com/gregr/papers/thesis.pdf
+    and Section 4.5 in vignette("gbm")
     *\param time survival times
     *\param event censoring indicate (event == 1 means dead)
     *\param f boosting fit
+    *\param w weights
 */
-SEXP ngradientCoxPLik(SEXP time, SEXP event, SEXP f) {
+SEXP ngradientCoxPLik(SEXP time, SEXP event, SEXP f, SEXP w) {
 
     SEXP ans;
-    double *dtime, *df, *dans, *dummy;
+    double *dtime, *df, *dans, *dummy, *dw;
     int *ievent, i, j, k, n;
     
     /* we don't assume the variables to be ordered w.r.t. time */
@@ -199,6 +201,7 @@ SEXP ngradientCoxPLik(SEXP time, SEXP event, SEXP f) {
     dtime = REAL(time);
     ievent = INTEGER(event);
     df = REAL(f);
+    dw = REAL(w);
     dummy = Calloc(n, double);
     
     for (i = 0; i < n; i++) {
@@ -208,15 +211,15 @@ SEXP ngradientCoxPLik(SEXP time, SEXP event, SEXP f) {
         
     for (j = 0; j < n; j++) {
         for (k = 0; k < n; k++) {
-            if (dtime[j] <= dtime[k])
-                dummy[j] += df[k];
+            if (!(dtime[j] > dtime[k]) || j == k)
+                dummy[j] += dw[k] * df[k];
         }
     }
     
     for (i = 0; i < n; i++) {
         for (j = 0; j < n; j++) {
-            if (ievent[j] & (dtime[j] <= dtime[i])) 
-                dans[i] += df[i] / dummy[j];
+            if (ievent[j] & (!(dtime[j] > dtime[i]))) 
+                dans[i] += dw[j] * df[i] / ((dummy[j] == 0) ? 1 : dummy[j]);
         }
         dans[i] = ievent[i] - dans[i];
     }
@@ -226,52 +229,6 @@ SEXP ngradientCoxPLik(SEXP time, SEXP event, SEXP f) {
     return(ans);
 }
 
-/**
-    partial likelihood of a Cox model
-    see formula (4.62) in Chapter 4 of Greg Ridgeway's thesis
-    http://www.i-pensieri.com/gregr/papers/thesis.pdf
-    *\param time survival times (ordered!)
-    *\param event censoring indicate (event == 1 means dead)
-    *\param f boosting fit
-*/
-SEXP CoxPLik(SEXP time, SEXP event, SEXP f) {
-
-    SEXP ans;
-    double *dtime, *df, *dans, *dummy, tmp;
-    int *ievent, i, j, n;
-    
-    /* we assume the variables to be ordered w.r.t. time */
-
-    /* allocate memory */
-    n = LENGTH(time);
-    PROTECT(ans = allocVector(REALSXP, 1));
-    dans = REAL(ans);
-    dtime = REAL(time);
-    ievent = INTEGER(event);
-    df = REAL(f);
-    dummy = Calloc(n, double);
-    
-    /* dummy[0] is total sum of exp(df) and 
-       dummy[j], j > 0 is abused as temp memory */
-    for (j = 0; j < n; j++) {
-        tmp = exp(df[j]);
-        dummy[0] += tmp;
-        if (j > 0)
-            dummy[j] = tmp;
-    }
-
-    for (j = 1; j < n; j++)
-        dummy[j] = dummy[j - 1] - dummy[j];
-    
-    dans[0] = 0.0;
-    for (i = 0; i < n; i++) 
-        dans[0] += ievent[i] * (df[i] - log(dummy[i]));
-        
-    Free(dummy);
-    UNPROTECT(1);
-    return(ans);
-}
-
 /**
     copy an R object
     *\param x an R object
@@ -309,3 +266,35 @@ SEXP wybar (SEXP ox, SEXP suox, SEXP tmp, SEXP ans) {
     }
     return(ans);
 }
+
+/**
+    partial likelihood of a Cox model
+    see formula (4.62) in Chapter 4 of Greg Ridgeway's thesis
+    http://www.i-pensieri.com/gregr/papers/thesis.pdf
+    and Section 4.5 in vignette("gbm")
+    *\param time survival times (ordered!)
+    *\param expf exp(boosting fit)
+*/
+
+SEXP R_risk (SEXP time, SEXP expf) {
+
+    SEXP ans;
+    double *dtime, *dexpf, *dans;
+    int i, j, n;
+    
+    n = LENGTH(time);
+    PROTECT(ans = allocVector(REALSXP, n));
+    dans = REAL(ans);
+    dtime = REAL(time);
+    dexpf = REAL(expf);
+    
+    for (i = 0; i < n; i++) {
+        dans[i] = 0;
+        for (j = 0; j < n; j++)
+            if (!(dtime[j] < dtime[i]) || j == i) 
+                dans[i] += dexpf[j];
+    }
+    
+    UNPROTECT(1);
+    return(ans);
+}
